@@ -4,6 +4,8 @@ use crate::error::{Error, ErrorKind};
 #[derive(Clone, Copy, Debug, Eq, Hash, PartialEq)]
 pub enum Proto {
     HTTPConnect,
+    SOCKS4,
+    SOCKS4A,
     SOCKS5,
 }
 
@@ -68,22 +70,28 @@ impl Proxy {
     /// * `proxy` - a str of format `<protocol>://<user>:<password>@<host>:port` . All parts except host are optional.
     /// # Protocols
     /// * `http`: HTTP Connect
-    /// * `socks`, `socks5`: SOCKS5 (requires socks feature)
+    /// * `socks4`: SOCKS4 (requires socks feature)
+    /// * `socks4a`: SOCKS4A (requires socks feature)
+    /// * `socks5` and `socks`: SOCKS5 (requires socks feature)
     /// # Examples
     /// * `http://127.0.0.1:8080`
     /// * `socks5://john:smith@socks.google.com`
     /// * `john:smith@socks.google.com:8000`
     /// * `localhost`
     pub fn new<S: AsRef<str>>(proxy: S) -> Result<Self, Error> {
-        let mut proxy_parts = proxy
-            .as_ref()
-            .splitn(2, "://")
-            .collect::<Vec<&str>>()
-            .into_iter();
+        let mut proxy = proxy.as_ref();
+
+        while proxy.ends_with('/') {
+            proxy = &proxy[..(proxy.len() - 1)];
+        }
+
+        let mut proxy_parts = proxy.splitn(2, "://").collect::<Vec<&str>>().into_iter();
 
         let proto = if proxy_parts.len() == 2 {
             match proxy_parts.next() {
                 Some("http") => Proto::HTTPConnect,
+                Some("socks4") => Proto::SOCKS4,
+                Some("socks4a") => Proto::SOCKS4A,
                 Some("socks") => Proto::SOCKS5,
                 Some("socks5") => Proto::SOCKS5,
                 _ => return Err(ErrorKind::InvalidProxyUrl.new()),
@@ -131,7 +139,7 @@ impl Proxy {
 
             match self.proto {
                 Proto::HTTPConnect => format!("Proxy-Authorization: basic {}\r\n", creds),
-                Proto::SOCKS5 => String::new(),
+                _ => String::new(),
             }
         } else {
             String::new()
@@ -190,6 +198,38 @@ mod tests {
         assert_eq!(proxy.proto, Proto::HTTPConnect);
     }
 
+    #[test]
+    fn parse_proxy_http_user_pass_server_port_trailing_slash() {
+        let proxy = Proxy::new("http://user:p@ssw0rd@localhost:9999/").unwrap();
+        assert_eq!(proxy.user, Some(String::from("user")));
+        assert_eq!(proxy.password, Some(String::from("p@ssw0rd")));
+        assert_eq!(proxy.server, String::from("localhost"));
+        assert_eq!(proxy.port, 9999);
+        assert_eq!(proxy.proto, Proto::HTTPConnect);
+    }
+
+    #[cfg(feature = "socks-proxy")]
+    #[test]
+    fn parse_proxy_socks4_user_pass_server_port() {
+        let proxy = Proxy::new("socks4://user:p@ssw0rd@localhost:9999").unwrap();
+        assert_eq!(proxy.user, Some(String::from("user")));
+        assert_eq!(proxy.password, Some(String::from("p@ssw0rd")));
+        assert_eq!(proxy.server, String::from("localhost"));
+        assert_eq!(proxy.port, 9999);
+        assert_eq!(proxy.proto, Proto::SOCKS4);
+    }
+
+    #[cfg(feature = "socks-proxy")]
+    #[test]
+    fn parse_proxy_socks4a_user_pass_server_port() {
+        let proxy = Proxy::new("socks4a://user:p@ssw0rd@localhost:9999").unwrap();
+        assert_eq!(proxy.user, Some(String::from("user")));
+        assert_eq!(proxy.password, Some(String::from("p@ssw0rd")));
+        assert_eq!(proxy.server, String::from("localhost"));
+        assert_eq!(proxy.port, 9999);
+        assert_eq!(proxy.proto, Proto::SOCKS4A);
+    }
+
     #[cfg(feature = "socks-proxy")]
     #[test]
     fn parse_proxy_socks_user_pass_server_port() {
